from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import input

import argparse
import sys

from publisher.git_handling.setup import setup_users_machine
from publisher.utils import (
    get_procedure_code,
    tspublisher_up_to_date,
    get_latest_tspub_version,
    get_current_tspub_version,
)
from publisher.translations.mo import build_mo_files
from publisher.translations.pot import ready_for_translations
from publisher.translations.po import merge_procedure_translation_files
from publisher import procedure_content
from publisher.exceptions import ValidationError
from publisher.processing import procedure
from publisher.processing.data_sources.ftrack import (
    FtrackAccountError,
    get_release_distribution_group,
    get_review_distribution_group,
)
from publisher.settings import CHANNEL_TYPES
from publisher.exceptions import ChannelExistsError, ChannelMissingError
from publisher.utils import get_yes_no


class PublisherCLI():

    def __init__(self, args):
        # Check version is up to date
        if not tspublisher_up_to_date():
            print("######################################################")
            print(
                "Your version of the tspub is out of date.\n"
                "You are using version {}\n"
                "The latest version is {}".format(
                    get_current_tspub_version(), get_latest_tspub_version()
                )
            )
            print("######################################################")

            if not get_yes_no(
                "Do you wish to continue with this outdated version? [y/N]: "
            ):
                sys.exit()

        parser = argparse.ArgumentParser(
            description='Touch Surgery Publisher',
            usage='tspub <command> [<args>]'
        )
        subparsers = parser.add_subparsers(
            title='Available commands',
            description=(
                'These commands allow you to create and '
                'publish TS simulations'
            ),
            help='subparsers help'
        )
        cli_commands = [c for c in dir(self) if not c.startswith('_')]

        for command in cli_commands:
            sub_parser = subparsers.add_parser(
                command,
                help=getattr(self, command).__doc__
            )
            sub_parser.set_defaults(func=getattr(self, command))

        namespace = parser.parse_args(args[0:1])

        try:
            namespace.func(args[1:])
        except ValidationError as e:
            print('The following issues were encountered.')
            print(e.message)

    def setup(self, args):
        """ Setup your machine, ready to create and edit simulations
        """
        setup_users_machine()

    def procedures(self, args):
        """ List all procedures in the git repository, each being a separate branch
        """
        procedure_content.display_procedures()

    def phases(self, args):
        """ List all phases of the procedure and their commit histories.
        """
        phase_code = procedure_content.get_user_selected_phase()
        procedure_content.display_phase_history(phase_code)

    def reset(self, args):
        """ Reset all local changes to match the most recent commit
        """
        procedure_content.cl_reset_repo()

    def revert_phase(self, args):
        """ Revert to a previous commit state for a chosen commit and phase.
        """
        phase_code = procedure_content.get_user_selected_phase()
        procedure_content.display_phase_history_and_revert(phase_code)

    def workon(self, args):
        """ Move to the branch containing the specified procedure so that it can be worked on
        """
        parser = argparse.ArgumentParser(
            description='Move to the branch containing the specified procedure',
            usage='''tspub workon <procedure_code>'''
        )
        parser.add_argument(
            'procedure',
            help='The procedure code'
        )

        self._clean_working_directory_state()

        args = parser.parse_args(args)
        procedure_content.change_procedure(args.procedure)

    def save(self, args):
        """ Commit and push current changes to the repo
        """
        parser = argparse.ArgumentParser(
            description='Commit and push current changes to the repo',
            usage='''tspub save <commit_message>'''
        )
        parser.add_argument(
            'commit_message',
            help='Describe what changes have been made'
        )
        args = parser.parse_args(args)
        procedure_content.save_working_changes(args.commit_message)

    def create(self, args):
        """Create a new branch with the name of the procedure_code param
        """
        parser = argparse.ArgumentParser(
            description='Create new procedure with the specified procedure_code param',
            usage='''tspub create <procedure_code>'''
        )
        parser.add_argument('procedure_code', help='code of the procedure to be created')
        args = parser.parse_args(args)

        if args.procedure_code not in procedure_content.build_procedure_list():
            self._clean_working_directory_state()
            procedure_content.create_procedure_branch(args.procedure_code)
        else:
            raise Exception("The procedure {0} already exists and thus cannot be created".format(args.procedure_code))

        procedure.create_procedure(args.procedure_code)

        procedure_content.save_working_changes("Initial commit", initial=True, procedure_code=args.procedure_code)

    def update_procedure(self, args):
        """Update current working procedure. Update phases with build_phases param.
        """
        parser = argparse.ArgumentParser(
            description='Update current procedure.',
            usage='''tspub update_procedure'''
        )
        parser.add_argument('--build_phases', action='store_true', help='Build all phases as well as procedure')
        parser.add_argument('--graphics', action='store_true', help='Copy latest images (normal)')
        parser.add_argument('--pip_graphics', action='store_true', help='Copy latest pip images (normal)')

        args = self._get_build_args(parser, args)

        procedure_code = get_procedure_code()

        graphics = args.graphics
        if args.graphics:
            graphics = "latest"

        pip_graphics = args.pip_graphics
        if args.pip_graphics:
            pip_graphics = {}

        try:
            procedure.build_procedure(procedure_code,
                                      build_phases=args.build_phases,
                                      graphics=graphics, pip_graphics=pip_graphics,
                                      widget_graphics=args.widget_graphics,
                                      thumbnails=args.thumbnails,
                                      step_numbers=args.step_numbers,
                                      info_step=args.info_step)
        except FtrackAccountError:
            raise RuntimeError(
                "update_procedure requires access to Ftrack "
                "and we were unable to find any ftrack account details"
            )

    def update_phase(self, args):
        """Update the specified phase with the phase_code param.
        """
        parser = argparse.ArgumentParser(
            description='Update specified phase with the phase_code param.',
            usage='''tspub update_phase <phase_code>'''
        )
        parser.add_argument('phase_code', help='code of the procedure to be created')
        parser.add_argument('--graphics', help='Copy latest images, can be supplied with version number',
                            const="latest", nargs="?")
        parser.add_argument('--pip_graphics', help='Copy latest pip images, can be supplied with version number e.g.: '
                                                   '--pip_graphics="pipName 2,otherPipName 1"', const={},
                            nargs="?")
        parser.add_argument('--csv_version', help='Define csv version to build with', nargs="+")

        args = self._get_build_args(parser, args)

        pip_graphics = self.format_pip_graphics(args.pip_graphics)

        if args.csv_version:
            csv_version = args.csv_version[0]
        else:
            csv_version = None

        try:
            procedure.build_single_phase(
                args.phase_code,
                graphics=args.graphics,
                pip_graphics=pip_graphics,
                widget_graphics=args.widget_graphics,
                thumbnails=args.thumbnails,
                step_numbers=args.step_numbers,
                info_step=args.info_step,
                csv_version=csv_version,
            )
        except FtrackAccountError:
            raise RuntimeError(
                "update_phase requires access to Ftrack "
                "and we were unable to find any ftrack account details"
            )

    def publish(self, args):
        """Publish procedure with the specified distribution group. Defaults to TS-Testing.
        """
        parser = argparse.ArgumentParser(
            description='Publish procedure with the specified distribution group. Defaults to TS-Testing.',
            usage='''tspub publish'''
        )
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--distgroup',
            help='Distribution group to be published to e.g. --distgroup=distGroup1,distGroup2',
            required=False
        )
        group.add_argument(
            "--review",
            action="store_true",
            help="Publish to review distribution group(s) specified in ftrack",
        )
        group.add_argument(
            "--release",
            action="store_true",
            help="Publish to release distribution group(s) specified in ftrack",
        )
        parser.add_argument(
            '--number',
            help='The number of previous commits to be shown',
            required=False
        )
        args = parser.parse_args(args)

        if args.review:
            try:
                distgroup = get_review_distribution_group(get_procedure_code())
            except FtrackAccountError:
                raise RuntimeError(
                    "Unable to use --review shortcut "
                    "as you are not setup to use Ftrack."
                    "\nPlease use --distgroup= method"
                )
            else:
                if not distgroup:
                    raise ValueError("No review groups specified in Ftrack")
        elif args.release:
            try:
                distgroup = get_release_distribution_group(get_procedure_code())
            except FtrackAccountError:
                raise RuntimeError(
                    "Unable to use --release shortcut "
                    "as you are not setup to use Ftrack.\n"
                    "Please use --distgroup= method"
                )
            else:
                if not distgroup:
                    raise ValueError("No release groups specified in Ftrack")
        else:
            distgroup = args.distgroup

        self._validate_distgroups(distgroup)

        procedure_content.publish_with_display(distgroup, args.number)

    def translate(self, args):
        """Available for testing only, builds a messages.pot in the working directory for the active procedure."""
        if ready_for_translations():
            merge_procedure_translation_files()
            build_mo_files()

    def generate_review_doc(self, args):
        if procedure_content.has_unstaged_changes():
            raise Exception(
                "You have unsaved local changes, "
                "please save or clear these before generating review document"
            )
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "--output_dir",
            help="Directory to output generated report to",
            required=False,
        )
        parser.add_argument(
            "--number",
            type=int,
            help="The number of previous commits to be shown",
            required=False,
        )
        args = parser.parse_args(args)
        procedure_content.generate_review_document_with_display(args.output_dir, args.number)

    def create_channel(self, args):
        parser = argparse.ArgumentParser()
        parser.add_argument('code')
        parser.add_argument('name')
        parser.add_argument('type', choices=CHANNEL_TYPES)
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--eula_required', dest='eula', action='store_true')
        group.add_argument('--eula_not_required', dest='eula', action='store_false')

        args = parser.parse_args(args)
        if args.code in procedure_content.build_channel_list():
            raise ChannelExistsError("A channel with code {} already exists.")
        procedure.build_channel(vars(args))

    def update_channel(self, args):
        parser = argparse.ArgumentParser()
        parser.add_argument('code')
        parser.add_argument('--name')
        parser.add_argument('--type', choices=CHANNEL_TYPES)
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--eula_required', dest='eula', action='store_true')
        group.add_argument('--eula_not_required', dest='eula', action='store_false')
        args = parser.parse_args(args)
        if args.code not in procedure_content.build_channel_list():
            raise ChannelMissingError("A channel with code {} does not exist.")

        updates = {}
        if args.name:
            updates['name'] = args.name
        if args.type:
            updates['type'] = args.type
        updates['eula'] = args.eula
        procedure.update_channel(args.code, updates)

    def save_channels(self, args):
        """ Commit and push current channel changes to the repo
        """
        parser = argparse.ArgumentParser(
            description='Commit and push current channel changes to the repo',
            usage='''tspub save_channels <commit_message>'''
        )
        parser.add_argument(
            'commit_message',
            help='Describe what changes have been made'
        )
        args = parser.parse_args(args)
        procedure_content.save_channel_changes(args.commit_message)

    @staticmethod
    def _get_build_args(parser, args):
        parser.add_argument('--widget_graphics', action='store_true', help='Copy latest widget images (normal)')
        parser.add_argument('--thumbnails', action='store_true', help="Pull latest thumbnails (both)")
        parser.add_argument('--step_numbers', action='store_true', help='Prepend step number in step content')
        parser.add_argument('--info_step', action='store_true', help='Add build information')

        args = parser.parse_args(args)
        return args

    @staticmethod
    def _clean_working_directory_state():
        if procedure_content.has_unstaged_changes():
            user_choice = PublisherCLI._get_user_decision()

            if user_choice == 'y':
                commit_message = PublisherCLI._get_commit_message()
                procedure_content.save_working_changes(commit_message)
            else:
                procedure_content.delete_unstaged_changes()

    @staticmethod
    def _get_commit_message():
        """ Return the commit message from the user
        """
        return input("Please enter a commit message")

    @staticmethod
    def _get_user_decision():
        """ Return the users y or n input
        """
        current_procedure = get_procedure_code()
        user_input = ''
        while user_input not in ['y', 'n']:
            print(
                "You have unsaved changes to {}. Would you save them now? "
                "If you do not save them they will be deleted".format(current_procedure)
            )
            print("> y or n:")
            sys.stdout.flush()
            user_input = input('')
        return user_input.lower()

    @staticmethod
    def _validate_distgroups(distgroups):
        if distgroups is None:
            return True

        groups = distgroups.split(",")

        if "Public" in groups and len(groups) != 1:
            raise ValueError(
                "Can not publish to Public and another distribution group "
                "at the same time"
            )
        elif "TS-Testing" in groups and len(groups) != 1:
            raise ValueError(
                "Can not publish to TS-Testing and another distribution group "
                "at the same time"
            )
        return True

    @staticmethod
    def format_pip_graphics(pip_versions):
        if pip_versions:
            pip_graphics = {}
            pips = pip_versions.split(",")
            for pip in pips:
                version_list = pip.split(" ")
                pip_graphics[version_list[0]] = version_list[1]
            return pip_graphics

        return pip_versions


def main():
    PublisherCLI(sys.argv[1:])



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import subprocess

import publisher.settings as settings
from publisher import settings as settings
from publisher.classes import Commit
from publisher.utils import (
    WorkingDirectory,
    call_command_and_print_exception,
    get_command_output,
    get_procedure_code,
)


def get_commits_for_procedure():
    """Get commits for a procedure.
    """
    return get_commits_for_repo()


def get_commits_for_phase(phase_code):
    """Get commits for a phase.
    """

    procedure_code = get_procedure_code()
    return get_commits_for_repo(relative_directory=os.path.join(procedure_code, phase_code))


def get_commits_for_repo(relative_directory=None):
    commit_list = []
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        subprocess.check_output(
            [
                "git",
                "-c",
                "protocol.version=2",
                "fetch",
                "--force",
                "--quiet",
                "origin",
                "refs/notes/*:refs/notes/*",
            ]
        )
        if relative_directory:
            output, error = get_command_output(['git', 'log', '--oneline', '--', relative_directory])
            log_list = output.split("\n")
        else:
            output, error = get_command_output(['git', 'log', '--oneline'])
            log_list = output.split("\n")

        for counter, log in enumerate(log_list):

            if log != '':
                commit_id = log.split()[0]
                comment = log.split(" ", 1)[1]
                note = get_commit_note(commit_id)
                author = get_author(commit_id)

                commit_list.append(Commit(commit_id, comment, author, note))

    return commit_list


def get_current_branch():
    command = ["git", "rev-parse", "--abbrev-ref", "HEAD"]
    branch, _ = get_command_output(command)
    return branch


def get_current_commit_id(short=True):
    command = ["git", "rev-parse"]
    if short:
        command.append("--short")
    command.append("HEAD")
    id, _ = get_command_output(command)
    return id

def get_author(commit_hash):
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        output, _ = get_command_output(['git', 'show', '--format="%aN <%aE>"', commit_hash])
        return output.split('"')[1]


def get_commit_note(commit_object):
    # Get the notes for the previous commits
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):

        try:
            note, error = get_command_output(['git', 'notes', 'show', commit_object])

            if 'no note found' not in error.lower():
                note = note.split('\n')[0]
            else:
                note = None

        except subprocess.CalledProcessError:
            note = None

    return note


class UsingCommit(object):
    def __init__(self, commit_id):
        self.commit_id = commit_id
        self.old_branch = None

    def __enter__(self):
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            self.old_branch = get_current_branch()
            current_commit = get_current_commit_id()
            if current_commit != self.commit_id:
                change_branch(self.commit_id)

    def __exit__(self, *_):
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            current_branch = get_current_branch()
            if current_branch != self.old_branch:
                change_branch(self.old_branch)


def change_branch(branch):
    command = [
        "git",
        "checkout",
        branch,
    ]
    message = (
        "Could not find the specified procedure. "
        "Make sure you have run setup and entered the correct "
        "procedure name"
    )
    print("Switching to: {}".format(branch))
    call_command_and_print_exception(command, message)

    print("Retrieving procedure assets")
    subprocess.run(["git", "-c", "protocol.version=2", "pull", "-q"])
    subprocess.run(["git", "clean", "-xdfq"])



from __future__ import absolute_import, division, print_function, unicode_literals

import os

import publisher.settings as settings
from publisher.processing.data_sources.utils import read_csv_file


def get_procedure_details(procedure_code, production_data):

    procedure_row = next((r for r in production_data if is_procedure_row(procedure_code, r)), None)
    if not procedure_row:
        raise EOFError("Could not find Procedure: %s in production csv." % procedure_code)
    procedure_name = procedure_row["Procedure Name"] or "%s - UNKNOWN" % procedure_code
    specialty_names = procedure_row["Speciality"].split("~") if procedure_row["Speciality"] else []
    specialties = [specialty_code_from_name(specialty) for specialty in specialty_names]
    channel = procedure_row["Channel"]
    vbs = procedure_row["VBS"]

    return procedure_name, specialties, channel, vbs


def get_procedure_phase_list(procedure_code, production_data):
    return [get_phase_details(r) for r in production_data if is_procedure_row(procedure_code, r)]


def get_phase_data(phase_code, production_data):
    return [get_phase_details(r) for r in production_data if is_phase_row(phase_code, r)]


def get_phase_details(r):
    return {"phase_code": r["Name"], 'phase_name': r['Module Name'], 'procedure_code': r["Procedure Code"],
            'order': r["Module Order"]}


def is_procedure_row(procedure_code, r):
    code = r.get("Procedure Code")
    return code == procedure_code


def is_phase_row(phase_code, r):
    code = r.get("Name")
    return code == phase_code


def get_production_csv_data(type="module"):
    csv_file = "{}.csv".format(type)
    if os.path.isdir(settings.BASE_DATA_DIR):
        module_csv = os.path.join(settings.PRODUCTION_INFO_DIR, csv_file)
    else:
        module_csv = os.path.join(settings.PRODUCTION_INFO_BACKUP_DIR, csv_file)

    headers, rows = read_csv_file(module_csv)
    return rows


def specialty_code_from_name(specialty):
    mappings = {
        "Anaesthetics": 'anaesthetics',
        "Cardiology": 'cardiology',
        "Cardiothoracic": 'cardiothoracic',
        "Dentistry": 'general_dentistry',
        "Ear, Nose & Throat": 'otorhinolaryngology',
        "Emergency Medicine": 'emergency_medicine',
        "General": 'general',
        "Neurosurgery": 'neurosurgery',
        "Obstetrics & Gynaecology": 'obstetrics_gynaecology',
        "Ophthalmic": 'ophthalmology',
        "Oral & Maxillofacial": 'oromaxillofacial',
        "Orthopaedics": 'orthopaedics_and_trauma',
        "Plastic & Reconstructive": 'plastic_reconstructive_aesthetic',
        "Urology": 'urology',
        "Vascular": 'vascular',
    }
    if specialty in mappings:
        return mappings[specialty]
    return specialty



from __future__ import absolute_import, division, print_function, unicode_literals

import os

import shutil

import publisher.settings as settings


def copy_eula_to_procedure(procedure_code, asset_directory):
    eula_file_name = "{0}_eula".format(procedure_code)

    source_file_path = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, procedure_code, "build_assets", "eula",
                                    "eula.txt")

    dest_file_path = os.path.join(asset_directory, eula_file_name + ".txt")

    if os.path.isfile(source_file_path):
        print("-- Found eula for procedure: %s --" % source_file_path)
        shutil.copyfile(source_file_path, dest_file_path)
        return eula_file_name

    return ""



from __future__ import division

from past.utils import old_div
from Qt import QtWidgets, QtCore


class CheckBoxDelegate(QtWidgets.QStyledItemDelegate):
    """
    A delegate that places a fully functioning QCheckBox in every
    cell of the column to which it's applied
    """

    def __init__(self, parent):
        super(CheckBoxDelegate, self).__init__(parent)

    def createEditor(self, parent, option, index):
        '''
        Important, otherwise an editor is created if the user clicks in this cell.
        ** Need to hook up a signal to the model
        '''
        return None

    def paint(self, painter, option, index):
        '''
        Paint a checkbox without the label.
        '''

        checked = index.data()
        check_box_style_option = QtWidgets.QStyleOptionButton()

        if (index.flags() & QtCore.Qt.ItemIsEditable) > 0:
            check_box_style_option.state |= QtWidgets.QStyle.State_Enabled
        else:
            check_box_style_option.state |= QtWidgets.QStyle.State_ReadOnly

        if checked:
            check_box_style_option.state |= QtWidgets.QStyle.State_On
        else:
            check_box_style_option.state |= QtWidgets.QStyle.State_Off

        check_box_style_option.rect = self.getCheckBoxRect(option)

        check_box_style_option.state |= QtWidgets.QStyle.State_Enabled

        QtWidgets.QApplication.style().drawControl(QtWidgets.QStyle.CE_CheckBox, check_box_style_option, painter)

    def editorEvent(self, event, model, option, index):
        '''
        Change the data in the model and the state of the checkbox
        if the user presses the left mousebutton or presses
        Key_Space or Key_Select and this cell is editable. Otherwise do nothing.
        '''
        if not (index.flags() & QtCore.Qt.ItemIsEditable) > 0:
            return False

        # Do not change the checkbox-state
        if event.type() == QtCore.QEvent.MouseButtonPress:
            return False
        if event.type() == QtCore.QEvent.MouseButtonRelease or event.type() == QtCore.QEvent.MouseButtonDblClick:
            if event.button() != QtCore.Qt.LeftButton or not self.getCheckBoxRect(option).contains(event.pos()):
                return False
            if event.type() == QtCore.QEvent.MouseButtonDblClick:
                return True
        elif event.type() == QtCore.QEvent.KeyPress:
            if event.key() != QtCore.Qt.Key_Space and event.key() != QtCore.Qt.Key_Select:
                return False
        else:
            return False

        # Change the checkbox-state
        self.setModelData(None, model, index)
        return True

    def setModelData(self, editor, model, index):
        '''
        The user wanted to change the old state in the opposite.
        '''
        newValue = not index.data()
        model.setData(index, newValue, QtCore.Qt.EditRole)

    def getCheckBoxRect(self, option):
        check_box_style_option = QtWidgets.QStyleOptionButton()
        check_box_rect = QtWidgets.QApplication.style().subElementRect(QtWidgets.QStyle.SE_CheckBoxIndicator,
                                                                       check_box_style_option, None)
        check_box_point = QtCore.QPoint(option.rect.x() +
                                        old_div(option.rect.width(), 2) -
                                        old_div(check_box_rect.width(), 2),
                                        option.rect.y() +
                                        old_div(option.rect.height(), 2) -
                                        old_div(check_box_rect.height(), 2))
        return QtCore.QRect(check_box_point, check_box_rect.size())


class ComboBoxDelegate(QtWidgets.QStyledItemDelegate):

    def __init__(self, parent, options):
        self._options = options
        super(ComboBoxDelegate, self).__init__(parent)

    def createEditor(self, parent, option, index):
        editor = QtWidgets.QComboBox(parent)
        editor.addItems(self._options)
        return editor

    def setEditorData(self, editor, index):
        editor.blockSignals(True)
        value = index.model().data(index, QtCore.Qt.EditRole)
        editor.setCurrentIndex(editor.findText(value))
        editor.blockSignals(False)

    def setModelData(self, editor, model, index):
        model.setData(index, editor.currentText(), QtCore.Qt.EditRole)



from __future__ import absolute_import, division, print_function

import csv
import os
import re
import six
from distutils.dir_util import copy_tree, remove_tree

from ruamel.yaml import YAML


class YamlObject(YAML):
    def __init__(self):
        YAML.__init__(self)
        self.default_flow_style = False
        self.block_seq_indent = 2
        self.indent = 4
        self.encoding = "utf-8"


def read_csv_file(csv_file):
    if six.PY2:
        with open(csv_file, "rbU") as f:
            reader = csv.DictReader(f, delimiter='|')

            rows = [
                {key: sanitise_new_lines(value) for key, value in row.items()}
                for row in reader]
            headers = reader.fieldnames
    else:
        with open(csv_file, "rU", encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter='|')

            rows = [{key: sanitise_new_lines(value) for key, value in row.items()} for row in reader]
            headers = reader.fieldnames

    return headers, rows


def sanitise_new_lines(string):
    if string is not None:
        string = string.replace("\r", "\n").rstrip("\n")
        string = string.decode("utf-8") if type(string) == bytes else string

    return string


def copy_assets(source_directory, destination_directory):
    if os.path.isdir(source_directory):
        copy_tree(source_directory, destination_directory)
        delete_misc_files(destination_directory)


def delete_misc_files(directory_name):
    thumbs_file = os.path.join(directory_name, "Thumbs.db")
    ds_store_file = os.path.join(directory_name, ".DS_Store")
    temp_folder = os.path.join(directory_name, "temp")

    if os.path.isfile(thumbs_file):
        os.remove(thumbs_file)
    if os.path.isfile(ds_store_file):
        os.remove(ds_store_file)
    if os.path.isdir(temp_folder):
        remove_tree(temp_folder)


def remove_whitespace_from_string(test):
    test = re.sub(r"(\n|\\n)+$", "", test)
    test = re.sub(r" {2,}", " ", test)
    test = re.sub(r"(\n|\\n){3,}", r"\n\n", test)
    return test



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'channel_editor.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from builtins import object
from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from publisher.gui.widgets import TSTitle
from publisher.gui.widgets import CustomGraphicsView


class Ui_channel_editor(object):
    def setupUi(self, channel_editor):
        if not channel_editor.objectName():
            channel_editor.setObjectName(u"channel_editor")
        channel_editor.resize(536, 425)
        self.add_row = QAction(channel_editor)
        self.add_row.setObjectName(u"add_row")
        self.delete_row = QAction(channel_editor)
        self.delete_row.setObjectName(u"delete_row")
        self.verticalLayout_2 = QVBoxLayout(channel_editor)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.header = QFrame(channel_editor)
        self.header.setObjectName(u"header")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.header.sizePolicy().hasHeightForWidth())
        self.header.setSizePolicy(sizePolicy)
        self.header.setFrameShape(QFrame.StyledPanel)
        self.header.setFrameShadow(QFrame.Raised)
        self.verticalLayout = QVBoxLayout(self.header)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label = TSTitle(self.header)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(20)
        self.label.setFont(font)

        self.verticalLayout.addWidget(self.label)

        self.line = QFrame(self.header)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)

        self.verticalLayout.addWidget(self.line)


        self.verticalLayout_2.addWidget(self.header)

        self.channel_tbl = QTableView(channel_editor)
        self.channel_tbl.setObjectName(u"channel_tbl")
        self.channel_tbl.setContextMenuPolicy(Qt.ActionsContextMenu)
        self.channel_tbl.setAlternatingRowColors(True)
        self.channel_tbl.setSortingEnabled(True)
        self.channel_tbl.horizontalHeader().setStretchLastSection(False)
        self.channel_tbl.verticalHeader().setVisible(False)

        self.verticalLayout_2.addWidget(self.channel_tbl)

        self.display_images = QCheckBox(channel_editor)
        self.display_images.setObjectName(u"display_images")

        self.verticalLayout_2.addWidget(self.display_images)

        self.badge_banner_widget = QWidget(channel_editor)
        self.badge_banner_widget.setObjectName(u"badge_banner_widget")
        self.gridLayout = QGridLayout(self.badge_banner_widget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName(u"gridLayout")
        self.badge = CustomGraphicsView(self.badge_banner_widget)
        self.badge.setObjectName(u"badge")

        self.gridLayout.addWidget(self.badge, 1, 0, 1, 1)

        self.banner = CustomGraphicsView(self.badge_banner_widget)
        self.banner.setObjectName(u"banner")

        self.gridLayout.addWidget(self.banner, 1, 1, 1, 1)

        self.label_3 = QLabel(self.badge_banner_widget)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)

        self.label_4 = QLabel(self.badge_banner_widget)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout.addWidget(self.label_4, 0, 1, 1, 1)


        self.verticalLayout_2.addWidget(self.badge_banner_widget)

        self.formLayout = QFormLayout()
        self.formLayout.setObjectName(u"formLayout")
        self.formLayout.setFieldGrowthPolicy(QFormLayout.AllNonFixedFieldsGrow)
        self.label_2 = QLabel(channel_editor)
        self.label_2.setObjectName(u"label_2")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label_2)

        self.msg = QPlainTextEdit(channel_editor)
        self.msg.setObjectName(u"msg")
        sizePolicy.setHeightForWidth(self.msg.sizePolicy().hasHeightForWidth())
        self.msg.setSizePolicy(sizePolicy)
        self.msg.setMaximumSize(QSize(16777215, 50))

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.msg)


        self.verticalLayout_2.addLayout(self.formLayout)

        self.submit_btn = QPushButton(channel_editor)
        self.submit_btn.setObjectName(u"submit_btn")

        self.verticalLayout_2.addWidget(self.submit_btn)

        QWidget.setTabOrder(self.channel_tbl, self.msg)
        QWidget.setTabOrder(self.msg, self.submit_btn)

        self.retranslateUi(channel_editor)

        QMetaObject.connectSlotsByName(channel_editor)
    # setupUi

    def retranslateUi(self, channel_editor):
        channel_editor.setWindowTitle(QCoreApplication.translate("channel_editor", u"Channel Editor", None))
        self.add_row.setText(QCoreApplication.translate("channel_editor", u"Add Row", None))
        self.delete_row.setText(QCoreApplication.translate("channel_editor", u"Delete Row", None))
        self.label.setText(QCoreApplication.translate("channel_editor", u"Channel Editor", None))
        self.display_images.setText(QCoreApplication.translate("channel_editor", u"Display Images", None))
        self.label_3.setText(QCoreApplication.translate("channel_editor", u"Badge", None))
        self.label_4.setText(QCoreApplication.translate("channel_editor", u"Banner", None))
        self.label_2.setText(QCoreApplication.translate("channel_editor", u"Comment", None))
        self.submit_btn.setText(QCoreApplication.translate("channel_editor", u"Save Changes", None))
    # retranslateUi




from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import input

import os
import requests
import subprocess
import sys

import publisher.settings as settings
from publisher.exceptions import SingleProceduresNotFoundError, ValidationError

DEFAULT_INPUT_READER = input


def get_content_directories(root_directory):

    def is_content_directory(d):
        ignored_directories = ['assets', 'translations', 'build_assets']
        return os.path.isdir(os.path.join(root_directory, d)) and d not in ignored_directories and not d.startswith('.')

    return [d for d in os.listdir(root_directory) if is_content_directory(d)]


class WorkingDirectory(object):
    """ Context manager for changing directory"""
    def __init__(self, new_dir):
        self.new_dir = new_dir
        self.old_dir = None

    def __enter__(self):
        self.old_dir = os.getcwd()
        os.chdir(self.new_dir)

    def __exit__(self, *_):
        os.chdir(self.old_dir)


def get_procedure_code():

    procedure_directories = get_content_directories(settings.PROCEDURE_CHECKOUT_DIRECTORY)

    if len(procedure_directories) != 1:
        raise SingleProceduresNotFoundError()

    return procedure_directories[0]


def get_user_commit_selection(message, objects, result_accessor=None):
    result_accessor = result_accessor or DEFAULT_INPUT_READER

    print("\n")
    for number, obj in enumerate(objects):
        print('{0}:'.format(number + 1), str(obj))

    print(message)
    sys.stdout.flush()

    user_input = int(result_accessor(''))

    if 1 <= user_input <= len(objects):
        return objects[user_input - 1]

    if user_input == 0:
        print("Operation cancelled.")
        sys.exit()

    else:
        print("Invalid option selected.")

    return None


def get_command_output(command_args):
    command = subprocess.Popen(command_args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, error = command.communicate()
    return output.decode("utf8").strip(), error.decode("utf8").strip()


def get_yes_no(message=''):
    user_response = DEFAULT_INPUT_READER(message)
    if user_response == 'y':
        return True
    return False


def get_input(message=''):
    user_response = DEFAULT_INPUT_READER(message)
    try:
        if int(user_response) == 0:
            print('Operation cancelled')
            sys.exit(1)
    except ValueError:
        return user_response


def get_platform():
    return sys.platform.lower()


def get_current_tspub_version():
    lines = get_command_output(['pip', 'show', 'tspublisher'])[0].split('\n')
    version_line = list((filter(lambda a: a.startswith('Version'), lines)))
    current_version = version_line[0].split()[1]
    return current_version


def get_latest_tspub_version():

    url = "https://pypi.org/pypi/tspublisher/json"
    response = requests.get(url)
    response.raise_for_status()
    data = response.json()
    latest_version = data["info"]["version"]

    return latest_version


def tspublisher_up_to_date():
    if get_current_tspub_version() == get_latest_tspub_version():
        return True
    return False


def find_command(cmd, path=None, pathext=None):
    """ Check if a command is on the PATH
    """
    if path is None:
        path = os.environ.get('PATH', '').split(os.pathsep)
    try:
        if isinstance(path, basestring):
            path = [path]
    except NameError:
        if isinstance(path, str):
            path = [path]
    # check if there are funny path extensions for executables, e.g. Windows
    if pathext is None:
        pathext = os.environ.get('PATHEXT', '.COM;.EXE;.BAT;.CMD').split(os.pathsep)
    # don't use extensions if the command ends with one of them
    for ext in pathext:
        if cmd.endswith(ext):
            pathext = ['']
            break
    # check if we find the command on PATH
    for p in path:
        f = os.path.join(p, cmd)
        if os.path.isfile(f):
            return f
        for ext in pathext:
            fext = f + ext
            if os.path.isfile(fext):
                return fext
    return None


def call_command_and_print_exception(command, message):
    try:
        return subprocess.check_output(command)
    except Exception as e:
        print(e)
        print('\n' + message)
        raise ValidationError



from __future__ import division

from past.utils import old_div
import imghdr
from Qt import QtWidgets, QtCore, QtGui

from publisher.gui.ui.ts_title import Ui_ts_title


class SpinnerOverlay(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super(SpinnerOverlay, self).__init__(parent)
        self._timer = QtCore.QTimer(self)
        self.box_size = 100
        self.thickness = 6
        self.arc_length = 300
        self._start_angle = 0
        self._timer.timeout.connect(self._on_timer_timeout)

        filter = ResizeEventFilter(parent)
        filter.resized.connect(self._on_parent_resized)
        parent.installEventFilter(filter)

        self.hide()

    def paintEvent(self, event):
        super(SpinnerOverlay, self).paintEvent(event)

        painter = QtGui.QPainter()
        painter.begin(self)
        try:
            painter.setRenderHint(QtGui.QPainter.Antialiasing)

            fg_col = self.palette().color(self.foregroundRole())
            pen = QtGui.QPen(fg_col)
            pen.setWidth(self.thickness)
            painter.setPen(pen)

            width_border = old_div((self.width()-self.box_size),2)
            height_border = old_div((self.height()-self.box_size),2)
            r = QtCore.QRect(0, 0, self.width(), self.height())
            r.adjust(width_border, height_border, -width_border, -height_border)

            # draw the arc:
            painter.drawArc(r, -self._start_angle * 16, self.arc_length * 16)
            r = None

        finally:
            painter.end()
            painter = None

    def showEvent(self, event):
        if not self._timer.isActive():
            self._timer.start(100)
        super(SpinnerOverlay, self).showEvent(event)

    def hideEvent(self, event):
        self._timer.stop()
        super(SpinnerOverlay, self).hideEvent(event)

    def closeEvent(self, event):
        self._timer.stop()
        super(SpinnerOverlay, self).closeEvent(event)

    def _on_timer_timeout(self):
        self._start_angle = (self._start_angle + 10)
        if self._start_angle > 360:
            self._start_angle -= 360
        self.update()

    def _on_parent_resized(self):
        self.resize(self.parentWidget().size())


class TSTitle(QtWidgets.QWidget, Ui_ts_title):

    def __init__(self, parent=None):
        super(TSTitle, self).__init__(parent)
        self.setupUi(self)

    def setText(self, text):
        self.label.setText(text)


class ResizeEventFilter(QtCore.QObject):
    """
    Event filter which emits a resized signal whenever
    the monitored widget resizes. This is so that the overlay wrapper
    class can be informed whenever the Widget gets a resize event.
    """
    resized = QtCore.Signal()

    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.Resize:
            self.resized.emit()
        return False


class CustomGraphicsView(QtWidgets.QGraphicsView):

    image_dropped = QtCore.Signal(str)

    def __init__(self, parent):
        super(CustomGraphicsView, self).__init__(parent)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dragLeaveEvent(self, event):
        event.accept()

    def dragMoveEvent(self, event):
        if event.mimeData().hasUrls:
            event.setDropAction(QtCore.Qt.CopyAction)
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        files = [url.toLocalFile() for url in event.mimeData().urls() if imghdr.what(url.toLocalFile())]
        if len(files) == 1:
            self.image_dropped.emit(files[0])
        elif len(files) > 1:
            self._message('Please select a single image file', 'Multiple files found')
        else:
            self._message('The selected file is not an image', 'Error')

    def _message(self, message, title):
        msg = QtWidgets.QMessageBox(self)
        msg.setIcon(QtWidgets.QMessageBox.Information)
        msg.setText('\n' + message + '\n')
        msg.setWindowTitle(title)
        msg.exec_()



import os
import imp

import ftrack_api
from ftrack_api.exception import NoResultFoundError

FTRACK_SECRETS_FILE = os.path.expanduser(os.path.join('~', 'ftrackSecrets.py'))


class FtrackAccountError(Exception):
    pass


def _get_ftrack_session():
    try:
        # Initially try creating session using environment variables
        session = ftrack_api.Session()
    except TypeError:
        # Fallback to details stored in ftrack secrets file
        try:
            ftrack_secrets = imp.load_source("ftrack_secrets", FTRACK_SECRETS_FILE)
        except IOError:
            raise FtrackAccountError("No Ftrack Account details found")
        else:
            session = ftrack_api.Session(
                server_url="https://touch-surgery.ftrackapp.com",
                api_key=ftrack_secrets.API_KEY,
                api_user=ftrack_secrets.API_USER,
            )
    return session


def get_procedure_details(procedure_code):

    session = _get_ftrack_session()

    try:
        procedure = session.query("Project where name is '{}'".format(procedure_code)).one()
    except NoResultFoundError:
        procedure = session.query("Procedure where name is '{}'".format(procedure_code)).one()

    custom_attributes = procedure['custom_attributes']
    specialties = custom_attributes['speciality'] if custom_attributes.get('speciality') else []
    channel = custom_attributes['institution'][0] if custom_attributes.get('institution') else ''
    labels = [i.strip() for i in custom_attributes.get("labels", "").split(",")]

    if isinstance(procedure, session.types['Procedure']):
        procedure_name = procedure['description']
        vbs = "VBS" in procedure['name']
    else:
        procedure_name = procedure['custom_attributes'].get('procedure_name')
        sim_type = custom_attributes['sim_type'][0] if custom_attributes.get('sim_type') else ''
        vbs = sim_type == 'vbs'

    return procedure_name, specialties, channel, labels, vbs


def get_procedure_phase_list(procedure_code):

    session = _get_ftrack_session()

    phases = session.query("select name, description, custom_attributes from Module where status.name is_not 'omit' "
                           "and project.name is '{}'".format(procedure_code)).all()
    if not phases:
        phases = session.query("select name, description, custom_attributes from Moduleold where status.name is_not "
                               "'omit' and ancestors any (name is '{}')".format(procedure_code)).all()

    phase_list = [_phase_detail(phase) for phase in phases]
    return sorted(phase_list, key=lambda p: (p["order"], p["phase_code"]))


def _get_geo_restriction(phase):
    geo_restrictions = phase["custom_attributes"]["geo_restrictions"]
    if geo_restrictions is None:
        if isinstance(phase, phase.session.types['Moduleold']):
            for ancestor in phase['ancestors']:
                if isinstance(ancestor, phase.session.types['Procedure']):
                    geo_restrictions = ancestor["custom_attributes"][
                        "geo_restrictions"]
                    break
        else:
            geo_restrictions = phase["project"]["custom_attributes"][
                "geo_restrictions"]
    return geo_restrictions or []


def _phase_detail(phase):
    if isinstance(phase, phase.session.types['Moduleold']):
        for ancestor in phase['ancestors']:
            if isinstance(ancestor, phase.session.types['Procedure']):
                procedure_code = ancestor['name']
                break
    else:
        procedure_code = phase['project']['name']

    return {
        "phase_code": phase['name'],
        "phase_name": phase['description'],
        "procedure_code": procedure_code,
        "order": phase['custom_attributes']['module_order'],
        "geo_restrictions": _get_geo_restriction(phase),
    }


def get_phase_data(phase_code):

    session = _get_ftrack_session()

    try:
        phase = session.query("select name, description, custom_attributes, project.name from Module where name is "
                              "'{}'".format(phase_code)).one()
    except NoResultFoundError:
        phase = session.query("select name, description, custom_attributes, ancestors.name from Moduleold where name is"
                              " '{}'".format(phase_code)).one()

    return _phase_detail(phase)


def get_release_distribution_group(procedure_code):

    session = _get_ftrack_session()
    try:
        procedure = session.query(
            "Project where name is '{}'".format(procedure_code)
        ).one()
    except NoResultFoundError:
        return ""
    custom_attributes = procedure["custom_attributes"]
    return custom_attributes.get("distgroup_release", "")


def get_review_distribution_group(procedure_code):

    session = _get_ftrack_session()
    try:
        procedure = session.query(
            "Project where name is '{}'".format(procedure_code)
        ).one()
    except NoResultFoundError:
        return ""
    custom_attributes = procedure["custom_attributes"]
    return custom_attributes.get("distgroup_review", "")




import operator
from Qt import QtCore


class ChannelTableModel(QtCore.QAbstractTableModel):

    validation_error = QtCore.Signal(str)

    def __init__(self, parent, *args):
        super(ChannelTableModel, self).__init__(parent, *args)
        self.mylist = []
        self.header = ['code', 'name', 'type', 'eula_required']

    def rowCount(self, parent):
        return len(self.mylist)

    def columnCount(self, parent):
        return len(self.header)

    def data(self, index, role):
        if not index.isValid():
            return None
        elif role == QtCore.Qt.EditRole:
            return self.mylist[index.row()][self.header[index.column()]]
        elif role != QtCore.Qt.DisplayRole:
            return None
        return self.mylist[index.row()][self.header[index.column()]]

    def setData(self, index, value, role):
        if not self.mylist[index.row()][self.header[index.column()]] == value:
            if self.header[index.column()] == 'code':
                value = value.upper()
                if not self._is_code_unique(value):
                    self.validation_error.emit("Code must be unique.\n"
                                               "Channel with code {} already exists.".format(value))
                    return False
            self.mylist[index.row()][self.header[index.column()]] = value
            self.mylist[index.row()]['changed'] = True
        return self.mylist[index.row()][self.header[index.column()]] == value

    def headerData(self, col, orientation, role):
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            return self.header[col]
        return None

    def flags(self, index):
        if not index.isValid():
            return QtCore.Qt.ItemIsEnabled
        if self.header[index.column()] == 'code' and not self.mylist[index.row()].get('new', False):
            return QtCore.QAbstractTableModel.flags(self, index)
        else:
            return QtCore.Qt.ItemFlags(QtCore.QAbstractTableModel.flags(self, index) | QtCore.Qt.ItemIsEditable)

    def sort(self, col, order):
        self.layoutAboutToBeChanged.emit()
        self.mylist.sort(key=operator.itemgetter(self.header[col]), reverse=(order == QtCore.Qt.DescendingOrder))
        self.layoutChanged.emit()

    def add_channel(self, data, row=None):
        self.layoutAboutToBeChanged.emit()
        if row:
            self.mylist.insert(row, data)
        else:
            self.mylist.append(data)
        self.layoutChanged.emit()

    def delete_row(self, row):
        if not self.mylist[row].get('new', False):
            self.validation_error.emit("Cannot delete channel with code {}".format(self.mylist[row].get('code')))
            return
        self.layoutAboutToBeChanged.emit()
        self.mylist.pop(row)
        self.layoutChanged.emit()

    @property
    def edited_channels(self):
        edited = []
        for row in self.mylist:
            if row.get('changed', False) or row.get('new', False):
                edited.append(row)
        return edited

    def _is_code_unique(self, value):
        for item in self.mylist:
            if item.get('code') == value:
                return False
        return True




import sys
from Qt import QtWidgets

from publisher.gui.channel_editor import ChannelEditor


def channel_editor():
    app = QtWidgets.QApplication(sys.argv)
    dialog = ChannelEditor()
    dialog.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    channel_editor()



# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 5.15.2
# WARNING! All changes made in this file will be lost!

from Qt import QtCompat, QtCore

qt_resource_data = b"\
\x00\x00\x0bf\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00 \x00\x00\x00 \x08\x06\x00\x00\x00szz\xf4\
\x00\x00\x00\x09pHYs\x00\x00\x0b\x13\x00\x00\x0b\x13\
\x01\x00\x9a\x9c\x18\x00\x00\x06\x98iTXtXML\
:com.adobe.xmp\x00\x00\
\x00\x00\x00<?xpacket beg\
in=\x22\xef\xbb\xbf\x22 id=\x22W5M\
0MpCehiHzreSzNTc\
zkc9d\x22?> <x:xmpm\
eta xmlns:x=\x22ado\
be:ns:meta/\x22 x:x\
mptk=\x22Adobe XMP \
Core 5.6-c142 79\
.160924, 2017/07\
/13-01:06:39    \
    \x22> <rdf:RDF \
xmlns:rdf=\x22http:\
//www.w3.org/199\
9/02/22-rdf-synt\
ax-ns#\x22> <rdf:De\
scription rdf:ab\
out=\x22\x22 xmlns:xmp\
=\x22http://ns.adob\
e.com/xap/1.0/\x22 \
xmlns:dc=\x22http:/\
/purl.org/dc/ele\
ments/1.1/\x22 xmln\
s:photoshop=\x22htt\
p://ns.adobe.com\
/photoshop/1.0/\x22\
 xmlns:xmpMM=\x22ht\
tp://ns.adobe.co\
m/xap/1.0/mm/\x22 x\
mlns:stEvt=\x22http\
://ns.adobe.com/\
xap/1.0/sType/Re\
sourceEvent#\x22 xm\
p:CreatorTool=\x22A\
dobe Photoshop C\
C (Windows)\x22 xmp\
:CreateDate=\x22201\
8-03-08T11:12:25\
Z\x22 xmp:ModifyDat\
e=\x222018-03-08T11\
:41:22Z\x22 xmp:Met\
adataDate=\x222018-\
03-08T11:41:22Z\x22\
 dc:format=\x22imag\
e/png\x22 photoshop\
:ColorMode=\x223\x22 p\
hotoshop:ICCProf\
ile=\x22sRGB IEC619\
66-2.1\x22 xmpMM:In\
stanceID=\x22xmp.ii\
d:48dadb43-ebb4-\
104e-9bdd-32a1de\
467c44\x22 xmpMM:Do\
cumentID=\x22adobe:\
docid:photoshop:\
8f60f140-c3d4-a3\
4c-87ce-b8474961\
44b4\x22 xmpMM:Orig\
inalDocumentID=\x22\
xmp.did:2a58b225\
-2e75-e94d-bdee-\
4030865c5daf\x22> <\
xmpMM:History> <\
rdf:Seq> <rdf:li\
 stEvt:action=\x22c\
reated\x22 stEvt:in\
stanceID=\x22xmp.ii\
d:2a58b225-2e75-\
e94d-bdee-403086\
5c5daf\x22 stEvt:wh\
en=\x222018-03-08T1\
1:12:25Z\x22 stEvt:\
softwareAgent=\x22A\
dobe Photoshop C\
C (Windows)\x22/> <\
rdf:li stEvt:act\
ion=\x22saved\x22 stEv\
t:instanceID=\x22xm\
p.iid:44467f4f-1\
dee-8541-bd3d-00\
2a4e7d424d\x22 stEv\
t:when=\x222018-03-\
08T11:14:34Z\x22 st\
Evt:softwareAgen\
t=\x22Adobe Photosh\
op CC (Windows)\x22\
 stEvt:changed=\x22\
/\x22/> <rdf:li stE\
vt:action=\x22saved\
\x22 stEvt:instance\
ID=\x22xmp.iid:48da\
db43-ebb4-104e-9\
bdd-32a1de467c44\
\x22 stEvt:when=\x2220\
18-03-08T11:41:2\
2Z\x22 stEvt:softwa\
reAgent=\x22Adobe P\
hotoshop CC (Win\
dows)\x22 stEvt:cha\
nged=\x22/\x22/> </rdf\
:Seq> </xmpMM:Hi\
story> </rdf:Des\
cription> </rdf:\
RDF> </x:xmpmeta\
> <?xpacket end=\
\x22r\x22?>\xfd\xf0\xcc\x85\x00\x00\x04tIDA\
TX\xc3\xa5\x97{\x88UU\x14\xc6\x7f\xf74\xe6\xe4\
(N\xcc\xa8e\x9ai\xaf\x89\x99\x1c\xcd\xd0\x18{\x11\
\xa5\x95Vb#\xf6P\xb2\xb2$($\xb50\x09\x0a\
\x0b\x22zL\xe5dDYVH\xa9=\xa6\xfc\xc3\x86\
\x094{`/g\xba\x96\xa5\x93\x1aj:j4e\
\x8d\x98\xa3\xdc\xfe\xf96|l\xce\xbd]q\xc1\xe1\xec\
\xbd\xce>{\xad\xbd\xbe\xf5\xda\x99Q-Y\x8a\xa4>\
\xc0\xb5\xc0X`\x040\x08\xe8\xabo\x07\x80]@\x16\
\xf8\x02X\x0dt\x16\xb3i\xa6\x08\x05\x86\x01\xf7\x037\
\x03\x15E*{\x00x\x07x\x0e\xf8\xe9x\x14X\x08\
<\x0cd\x22\xfe\x16\xe0\x17\xa0\x03\xc8\x01\x03\x803\x81\
\xf3R\xf6x\x0ax0\x9f\x80\x92<\xfcS\x81&`\
\xb4\xf1\xbe\x05\x96\x02\xcd\x12\x9eFC\x81\xab\x80\xdb\x80\
:\xf1\x1e\x00\xae\x06n\x00\xb6\x17c\x81a\xc0z\xa0\
\x9f\xe6\xbb\x80y\xc0r\x8e\x8d\xae\x07\x9e\x01\xce\xd2\xfc\
\x1f\xf9O\xb6\x90\x02\x15@;p\xb2\xe6M\xc0\xad\xc0\
A[s90\x0e\x18\x0eT\x8a\xb7\x1f\xf8\x0ex\x01\
\xf8#\xb2\xf0R\xed\x01p\x088[\x87J\x85`\xad\
\x09_\x02\xcc\xb4oS\x80\x87\x80\x91yN<\x11\xa8\
\x01\xea\x8dw\x04\x98&_\x99\x0b\x94\x02\x9f\x09*\x00\
\x92\xc8\xe1j4^\x1d\x09_\x06\xac\x88\x84\xef\x00>\
\xd5\xb3O\xc2\xd6\xe6Qn\x1e\xf0\xb6\xc6g\x00\x8bb\
\x08\x06\x02\xbf\x89\xd7)(r\x9a\xaf\x03.\xb1\xcd\x1a\
\x80\xb7\x806[S\x0a\x94\xeb\xa4\x08\xf7\xe5\x82\xe3F\
\x85%\xc0N\xe5\x0f\x145\xdb\x82\x05\x1e1\x013l\
\xe3\x15&|\x13P\x0d\xcc\x01ZmM\xc0\xb6\xc3\xe6\
\xe3\x80\x0b\x80+\xcd\x091_\x00x\x22X\xa0\x17\xf0\
'\xd0\x03\xd8\x0cT\x19\xa6\xab4n\x97\xf0n\xcd\xcb\
\x80\xc9\x12\x92\x915>\x00\xfe\xd2\xf7r\x99\xb9S\x0a\
\x1f1\xc1_YxW\x96(\xbd\xf60\xf3\x06Zl\
\xe3+L\xf8\x14\xa0\x11\xe8\x1f\xe1\xfc\xb4\x1c\xed\x0d\x1d\
hz\x1e\x7fxVY\x12`R\x223\x05\xfaH\xef\
\xcb\x80\xc1\x1a/\xb2\xb0\x99(X\x82\xf0\xa3\xa6X\x85\
B\xee\x96H\xe0@A\x829\xf8\xd1p\xb0\xc4<{\
\x0b\xb0G\xe3\xeb\xec\x87\xe7\xf5\xee\x05\xbco\xfc\xf9\xc0\
\xe9z\xe6D\x11SnP\xb5*{\xdemu\xa2U\
\xe3\xdaD\x1a\x02\xfc`\x9b\xd4Z\xa8m\xd5\xf8&\x83\
j\x01\xf0$\xb0[\xce\xd7\x00\xdck\xff\xdfna\x1e\
2\xead\xfb\xfe\xb3\xde\xfd\x13\x95Y\xa2\x0c\x162\xdc\
\xaf\xc6\x0b\xb9\xfd_\xe1\x1d\xd3\x8b\xc2\x1e\xe0B\xbd\xff\
V\x0d\xc1\xac\x122'@\xcf\xe4\x18r{8\xfd\x89\
\xaa~\x05\xabl\x0a/\x976N\xa4%\x96\x82\x01~\
\xb7\xea\x16\xe8\x13\xdb\xfc\xa4\x14\x013\xed\x94\x1blm\
\xc8\x03{lm\x80\xe5p\x89p\x1c\xa48\x0f\xd4\xa6\
\xe8\x18\xac\xe2\xd1\xae\xec\xd7\x17\xd8\xab\xb9\xc3U\x0f\xbc\
d\xbc\xd7\xf4\x1ei\x07\xf3\xaaw\xae\xde\xfb\x12\xe0{\
M\xaa\xcc\xb4\x1f\xda\xe2\xb96n\x04Vj<A>\
\xb23\x12~\x87\xf9\x93GG\xd8\xb3\xb7E^61\
\xd3\x868\x07\xf8\xdc\x9a\x87Y\xea\x11b\xaa\x07\x86\xa8\
\x0e\x00t\x01\xf7\x00\xafk~\xbe\xa5\xde\x8d\x16z\xe3\
\xcd\x9f\xd6dF\xb5d\xcb\x94BO\x00~\x94v\xdd\
Z\xf8\xb1\x15\x91j\xf3\x17\x80ST\x9e\xcb\x84\xf9{\
\x82'\xc0\xb2\xc9\xb0\xbeX\xcd*z\xd7y\x18v\x01\
o\x8aQ\x0d\x9c\xa3q\xb3pG\xbe\xd0\x16\x95\xe3\x0e\
`\xb6\x9co\xb1\x09\xaf\xd3\xda~\x96\xd2\x83\xf0\x8bL\
x\x13\xb0?\x94\xe3J\x9d\xb6\x03\x98*\xa5\x02\xb5D\
\xe9\xfae)\xf6\x0dp8\xc4\xb36\x9f\xa1'\xd0*\
\xb5f\x81\xb6YdU\x01\x9b3E\xde\x0b\x96\xc8\xb9\
\x9c\xf6Z\xa2\x1a\x9aR\x9c\x1a\x81\xfb\xa2\xc8\x08\x19\xf2\
U\xe0\xae\xb8#\xf2\x84\xf3h\xe4\xfdw\xaa\xab]o\
\xbc\x01\xc0\x18=.|\x9d\xba`\x17\xfe\xb8\x09\xdfm\
u!\xb5+\x9ef\xd8\x8f\x05\xbe\x8c\xbe\x8f\x96\x83\xd6\
\xa8}\x0f\x9bf\x05\xe3\x86h\xfd+\xd6\xde\xe5\x94W\
\xb6\x16\xba\x17d\xd5\xe3\x1dT1\x8a\xe9k=\xffG\
\xe3U3j\xacA\xbd\xd4\x85\x17R`\x88\x8aNN\
\xe9t\x81B\xe9]9\xe5\x8e<BO\xd3\xc5d\xba\
\x9a\x18\xbfIMJ\xbb\xa6\x15\xe3\x84}\xac\xa9\x0cf\
\xcc*\x1d\xc7W\xb3\x11\xca'q\x95\x9cmMHQ\
W3\xa7.\xe01\xa5\xd52Y\xa4\xd6z\x864\xea\
V\xe7\xd4\xa0\x0b\xcbq\xdd\x8e\xbd\x82]#\x1c\x87\xab\
\x13*\x95B\x87\xd4\xb6mT\x144[\x9b_\x90\xfe\
\x03=x\x1ffGD\xffn\x00\x00\x00\x00IEN\
D\xaeB`\x82\
"

qt_resource_name = b"\
\x00\x05\
\x00o\xa6S\
\x00i\
\x00c\x00o\x00n\x00s\
\x00\x0a\
\x0ajx\xa7\
\x00T\
\x00S\x00I\x00c\x00o\x00n\x00.\x00p\x00n\x00g\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x03\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x10\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01pR\xb6\xfcv\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



from __future__ import absolute_import, division, print_function, unicode_literals
import requests


class TouchSurgery(object):

    def __init__(self):
        self.auth_token = None

    def authorized(self):
        return bool(self.auth_token)

    def login(self, email, password):
        response = requests.post(
            'https://live.touchsurgery.com/api/auth/oauth2/token/',
            data=dict(
                grant_type="password",
                username=email,
                password=password,
                client_id="ePtPFBu8hjRVuf2dlyQDwlB5wQiqfFH4i1OjMhYh",
            )
        )

        if response.status_code != 200:
            print('\n' + 'Error code: ' + str(response.status_code))
            print('Your username/password combination was not recognised, please try again. (Enter 0 to cancel)' + '\n')
            return False

        payload = response.json()
        self.auth_token = payload['access_token']
        return True

    def upload_key(self, key):
        response = requests.post(
            'https://live.touchsurgery.com/api/v3/access-key',
            data=dict(public_key=key),
            headers=dict(Authorization='bearer {0}'.format(self.auth_token))
        )
        if response.status_code != 201:
            print('\n' + 'Error code: ' + str(response.status_code))
            print('Your rsa key was not recognised, please try again or contact pipeline.')
            return False
        return True
